package emulator

var (
	JOYPADIO  uint16 = 0xff00
	SBIO      uint16 = 0xff01
	SCIO      uint16 = 0xff02
	DIVIO     uint16 = 0xff04
	TIMAIO    uint16 = 0xff05
	TMAIO     uint16 = 0xff06
	TACIO     uint16 = 0xff07
	IFIO      uint16 = 0xff0f
	LCDCIO    uint16 = 0xff40
	LCDSTATIO uint16 = 0xff41
	LYIO      uint16 = 0xff44
	DMAIO     uint16 = 0xff46
	BGPIO     uint16 = 0xff47
	OBP0IO    uint16 = 0xff48
	OBP1IO    uint16 = 0xff49
	WYIO      uint16 = 0xff4a
	WXIO      uint16 = 0xff4b
	KEY1IO    uint16 = 0xff4d
	VBKIO     uint16 = 0xff4f
	HDMA1IO   uint16 = 0xff51
	HDMA2IO   uint16 = 0xff52
	HDMA3IO   uint16 = 0xff53
	HDMA4IO   uint16 = 0xff54
	HDMA5IO   uint16 = 0xff55
	BCPSIO    uint16 = 0xff68
	BCPDIO    uint16 = 0xff69
	OCPSIO    uint16 = 0xff6a
	OCPDIO    uint16 = 0xff6b
	SVBKIO    uint16 = 0xff70
	IEIO      uint16 = 0xffff

	instructions [256][4]string = [256][4]string{
		/* 0x0x */ {"NOP", "*", "*", "1"}, {"LD", "BC", "d16", "3"}, {"LD", "(BC)", "A", "2"}, {"INC", "BC", "*", "2"}, {"INC", "B", "*", "1"}, {"DEC", "B", "*", "1"}, {"LD", "B", "d8", "2"}, {"RLCA", "*", "*", "1"}, {"LD", "(a16)", "SP", "5"}, {"ADD", "HL", "BC", "2"}, {"LD", "A", "(BC)", "2"}, {"DEC", "BC", "*", "2"}, {"INC", "C", "*", "1"}, {"DEC", "C", "*", "1"}, {"LD", "C", "d8", "2"}, {"RRCA", "*", "*", "1"},
		/* 0x1x */ {"STOP", "0", "*", "1"}, {"LD", "DE", "d16", "3"}, {"LD", "(DE)", "A", "2"}, {"INC", "DE", "*", "2"}, {"INC", "D", "*", "1"}, {"DEC", "D", "*", "1"}, {"LD", "D", "d8", "2"}, {"RLA", "*", "*", "1"}, {"JR", "r8", "*", "3"}, {"ADD", "HL", "DE", "2"}, {"LD", "A", "(DE)", "2"}, {"DEC", "DE", "*", "2"}, {"INC", "E", "*", "1"}, {"DEC", "E", "*", "1"}, {"LD", "E", "d8", "2"}, {"RRA", "*", "*", "1"},
		/* 0x2x */ {"JR", "NZ", "r8", "1.5"}, {"LD", "HL", "d16", "3"}, {"LD", "(HL+)", "A", "2"}, {"INC", "HL", "*", "2"}, {"INC", "H", "*", "1"}, {"DEC", "H", "*", "1"}, {"LD", "H", "d8", "2"}, {"DAA", "*", "*", "1"}, {"JR", "Z", "r8", "1.5"}, {"ADD", "HL", "HL", "2"}, {"LD", "A", "(HL+)", "2"}, {"DEC", "HL", "*", "2"}, {"INC", "L", "*", "1"}, {"DEC", "L", "*", "1"}, {"LD", "L", "d8", "2"}, {"CPL", "*", "*", "1"},
		/* 0x3x */ {"JR", "NC", "r8", "1.5"}, {"LD", "SP", "d16", "3"}, {"LD", "(HL-)", "A", "2"}, {"INC", "SP", "*", "2"}, {"INC", "(HL)", "*", "3"}, {"DEC", "(HL)", "*", "3"}, {"LD", "(HL)", "d8", "3"}, {"SCF", "*", "*", "1"}, {"JR", "C", "r8", "1.5"}, {"ADD", "HL", "SP", "2"}, {"LD", "A", "(HL-)", "2"}, {"DEC", "SP", "*", "2"}, {"INC", "A", "*", "1"}, {"DEC", "A", "*", "1"}, {"LD", "A", "d8", "2"}, {"CCF", "*", "*", "1"},
		/* 0x4x */ {"LD", "B", "B", "1"}, {"LD", "B", "C", "1"}, {"LD", "B", "D", "1"}, {"LD", "B", "E", "1"}, {"LD", "B", "H", "1"}, {"LD", "B", "L", "1"}, {"LD", "B", "(HL)", "2"}, {"LD", "B", "A", "1"}, {"LD", "C", "B", "1"}, {"LD", "C", "C", "1"}, {"LD", "C", "D", "1"}, {"LD", "C", "E", "1"}, {"LD", "C", "H", "1"}, {"LD", "C", "L", "1"}, {"LD", "C", "(HL)", "2"}, {"LD", "C", "A", "1"},
		/* 0x5x */ {"LD", "D", "B", "1"}, {"LD", "D", "C", "1"}, {"LD", "D", "D", "1"}, {"LD", "D", "E", "1"}, {"LD", "D", "H", "1"}, {"LD", "D", "L", "1"}, {"LD", "D", "(HL)", "2"}, {"LD", "D", "A", "1"}, {"LD", "E", "B", "1"}, {"LD", "E", "C", "1"}, {"LD", "E", "D", "1"}, {"LD", "E", "E", "1"}, {"LD", "E", "H", "1"}, {"LD", "E", "L", "1"}, {"LD", "E", "(HL)", "2"}, {"LD", "E", "A", "1"},
		/* 0x6x */ {"LD", "H", "B", "1"}, {"LD", "H", "C", "1"}, {"LD", "H", "D", "1"}, {"LD", "H", "E", "1"}, {"LD", "H", "H", "1"}, {"LD", "H", "L", "1"}, {"LD", "H", "(HL)", "2"}, {"LD", "H", "A", "1"}, {"LD", "L", "B", "1"}, {"LD", "L", "C", "1"}, {"LD", "L", "D", "1"}, {"LD", "L", "E", "1"}, {"LD", "L", "H", "1"}, {"LD", "L", "L", "1"}, {"LD", "L", "(HL)", "2"}, {"LD", "L", "A", "1"},
		/* 0x7x */ {"LD", "(HL)", "B", "2"}, {"LD", "(HL)", "C", "2"}, {"LD", "(HL)", "D", "2"}, {"LD", "(HL)", "E", "2"}, {"LD", "(HL)", "H", "2"}, {"LD", "(HL)", "L", "2"}, {"HALT", "*", "*", "1"}, {"LD", "(HL)", "A", "2"}, {"LD", "A", "B", "1"}, {"LD", "A", "C", "1"}, {"LD", "A", "D", "1"}, {"LD", "A", "E", "1"}, {"LD", "A", "H", "1"}, {"LD", "A", "L", "1"}, {"LD", "A", "(HL)", "2"}, {"LD", "A", "A", "1"},
		/* 0x8x */ {"ADD", "A", "B", "1"}, {"ADD", "A", "C", "1"}, {"ADD", "A", "D", "1"}, {"ADD", "A", "E", "1"}, {"ADD", "A", "H", "1"}, {"ADD", "A", "L", "1"}, {"ADD", "A", "(HL)", "2"}, {"ADD", "A", "A", "1"}, {"ADC", "A", "B", "1"}, {"ADC", "A", "C", "1"}, {"ADC", "A", "D", "1"}, {"ADC", "A", "E", "1"}, {"ADC", "A", "H", "1"}, {"ADC", "A", "L", "1"}, {"ADC", "A", "(HL)", "2"}, {"ADC", "A", "A", "1"},
		/* 0x9x */ {"SUB", "B", "*", "1"}, {"SUB", "C", "*", "1"}, {"SUB", "D", "*", "1"}, {"SUB", "E", "*", "1"}, {"SUB", "H", "*", "1"}, {"SUB", "L", "*", "1"}, {"SUB", "(HL)", "*", "2"}, {"SUB", "A", "*", "1"}, {"SBC", "A", "B", "1"}, {"SBC", "A", "C", "1"}, {"SBC", "A", "D", "1"}, {"SBC", "A", "E", "1"}, {"SBC", "A", "H", "1"}, {"SBC", "A", "L", "1"}, {"SBC", "A", "(HL)", "2"}, {"SBC", "A", "A", "1"},
		/* 0xax */ {"AND", "B", "*", "1"}, {"AND", "C", "*", "1"}, {"AND", "D", "*", "1"}, {"AND", "E", "*", "1"}, {"AND", "H", "*", "1"}, {"AND", "L", "*", "1"}, {"AND", "(HL)", "*", "2"}, {"AND", "A", "*", "1"}, {"XOR", "B", "*", "1"}, {"XOR", "C", "*", "1"}, {"XOR", "D", "*", "1"}, {"XOR", "E", "*", "1"}, {"XOR", "H", "*", "1"}, {"XOR", "L", "*", "1"}, {"XOR", "(HL)", "*", "2"}, {"XOR", "A", "*", "1"},
		/* 0xbx */ {"OR", "B", "*", "1"}, {"OR", "C", "*", "1"}, {"OR", "D", "*", "1"}, {"OR", "E", "*", "1"}, {"OR", "H", "*", "1"}, {"OR", "L", "*", "1"}, {"OR", "(HL)", "*", "2"}, {"OR", "A", "*", "1"}, {"CP", "B", "*", "1"}, {"CP", "C", "*", "1"}, {"CP", "D", "*", "1"}, {"CP", "E", "*", "1"}, {"CP", "H", "*", "1"}, {"CP", "L", "*", "1"}, {"CP", "(HL)", "*", "2"}, {"CP", "A", "*", "1"},
		/* 0xcx */ {"RET", "NZ", "*", "2.5"}, {"POP", "BC", "*", "3"}, {"JP", "NZ", "a16", "1.333"}, {"JP", "a16", "*", "4"}, {"CALL", "NZ", "a16", "2"}, {"PUSH", "BC", "*", "4"}, {"ADD", "A", "d8", "2"}, {"RST", "00", "*", "4"}, {"RET", "Z", "*", "2.5"}, {"RET", "*", "*", "4"}, {"JP", "Z", "a16", "1.333"}, {"PREFIX CB", "*", "*", "1"}, {"CALL", "Z", "a16", "2"}, {"CALL", "a16", "*", "6"}, {"ADC", "A", "d8", "2"}, {"RST", "08", "*", "4"},
		/* 0xdx */ {"RET", "NC", "*", "2.5"}, {"POP", "DE", "*", "3"}, {"JP", "NC", "a16", "1.333"}, {"*", "*", "*", "*"}, {"CALL", "NC", "a16", "2"}, {"PUSH", "DE", "*", "4"}, {"SUB", "d8", "*", "2"}, {"RST", "16", "*", "4"}, {"RET", "C", "*", "2.5"}, {"RETI", "*", "*", "4"}, {"JP", "C", "a16", "1.333"}, {"*", "*", "*", "*"}, {"CALL", "C", "a16", "2"}, {"*", "*", "*", "*"}, {"SBC", "A", "d8", "2"}, {"RST", "24", "*", "4"},
		/* 0xex */ {"LDH", "(a8)", "A", "3"}, {"POP", "HL", "*", "3"}, {"LD", "(C)", "A", "2"}, {"*", "*", "*", "*"}, {"*", "*", "*", "*"}, {"PUSH", "HL", "*", "4"}, {"AND", "d8", "*", "2"}, {"RST", "32", "*", "4"}, {"ADD", "SP", "r8", "4"}, {"JP", "(HL)", "*", "1"}, {"LD", "(a16)", "A", "4"}, {"*", "*", "*", "*"}, {"*", "*", "*", "*"}, {"*", "*", "*", "*"}, {"XOR", "d8", "*", "2"}, {"RST", "40", "*", "4"},
		/* 0xfx */ {"LDH", "A", "(a8)", "3"}, {"POP", "AF", "*", "3"}, {"LD", "A", "(C)", "2"}, {"DI", "*", "*", "1"}, {"*", "*", "*", "*"}, {"PUSH", "AF", "*", "4"}, {"OR", "d8", "*", "2"}, {"RST", "48", "*", "4"}, {"LD", "HL", "SP+r8", "3"}, {"LD", "SP", "HL", "2"}, {"LD", "A", "(a16)", "4"}, {"EI", "*", "*", "1"}, {"*", "*", "*", "*"}, {"*", "*", "*", "*"}, {"CP", "d8", "*", "2"}, {"RST", "56", "*", "4"},
	}

	prefixCB [256][4]string = [256][4]string{
		/* 0x0x */ {"RLC", "B", "*", "2"}, {"RLC", "C", "*", "2"}, {"RLC", "D", "*", "2"}, {"RLC", "E", "*", "2"}, {"RLC", "H", "*", "2"}, {"RLC", "L", "*", "2"}, {"RLC", "(HL)", "*", "4"}, {"RLC", "A", "*", "2"}, {"RRC", "B", "*", "2"}, {"RRC", "C", "*", "2"}, {"RRC", "D", "*", "2"}, {"RRC", "E", "*", "2"}, {"RRC", "H", "*", "2"}, {"RRC", "L", "*", "2"}, {"RRC", "(HL)", "*", "4"}, {"RRC", "A", "*", "2"},
		/* 0x1x */ {"RL", "B", "*", "2"}, {"RL", "C", "*", "2"}, {"RL", "D", "*", "2"}, {"RL", "E", "*", "2"}, {"RL", "H", "*", "2"}, {"RL", "L", "*", "2"}, {"RL", "(HL)", "*", "4"}, {"RL", "A", "*", "2"}, {"RR", "B", "*", "2"}, {"RR", "C", "*", "2"}, {"RR", "D", "*", "2"}, {"RR", "E", "*", "2"}, {"RR", "H", "*", "2"}, {"RR", "L", "*", "2"}, {"RR", "(HL)", "*", "4"}, {"RR", "A", "*", "2"},
		/* 0x2x */ {"SLA", "B", "*", "2"}, {"SLA", "C", "*", "2"}, {"SLA", "D", "*", "2"}, {"SLA", "E", "*", "2"}, {"SLA", "H", "*", "2"}, {"SLA", "L", "*", "2"}, {"SLA", "(HL)", "*", "4"}, {"SLA", "A", "*", "2"}, {"SRA", "B", "*", "2"}, {"SRA", "C", "*", "2"}, {"SRA", "D", "*", "2"}, {"SRA", "E", "*", "2"}, {"SRA", "H", "*", "2"}, {"SRA", "L", "*", "2"}, {"SRA", "(HL)", "*", "4"}, {"SRA", "A", "*", "2"},
		/* 0x3x */ {"SWAP", "B", "*", "2"}, {"SWAP", "C", "*", "2"}, {"SWAP", "D", "*", "2"}, {"SWAP", "E", "*", "2"}, {"SWAP", "H", "*", "2"}, {"SWAP", "L", "*", "2"}, {"SWAP", "(HL)", "*", "4"}, {"SWAP", "A", "*", "2"}, {"SRL", "B", "*", "2"}, {"SRL", "C", "*", "2"}, {"SRL", "D", "*", "2"}, {"SRL", "E", "*", "2"}, {"SRL", "H", "*", "2"}, {"SRL", "L", "*", "2"}, {"SRL", "(HL)", "*", "4"}, {"SRL", "A", "*", "2"},

		/* 0x4x */ {"BIT", "0", "B", "2"}, {"BIT", "0", "C", "2"}, {"BIT", "0", "D", "2"}, {"BIT", "0", "E", "2"}, {"BIT", "0", "H", "2"}, {"BIT", "0", "L", "2"}, {"BIT", "0", "(HL)", "4"}, {"BIT", "0", "A", "2"}, {"BIT", "1", "B", "2"}, {"BIT", "1", "C", "2"}, {"BIT", "1", "D", "2"}, {"BIT", "1", "E", "2"}, {"BIT", "1", "H", "2"}, {"BIT", "1", "L", "2"}, {"BIT", "1", "(HL)", "4"}, {"BIT", "1", "A", "2"},
		/* 0x5x */ {"BIT", "2", "B", "2"}, {"BIT", "2", "C", "2"}, {"BIT", "2", "D", "2"}, {"BIT", "2", "E", "2"}, {"BIT", "2", "H", "2"}, {"BIT", "2", "L", "2"}, {"BIT", "2", "(HL)", "4"}, {"BIT", "2", "A", "2"}, {"BIT", "3", "B", "2"}, {"BIT", "3", "C", "2"}, {"BIT", "3", "D", "2"}, {"BIT", "3", "E", "2"}, {"BIT", "3", "H", "2"}, {"BIT", "3", "L", "2"}, {"BIT", "3", "(HL)", "4"}, {"BIT", "3", "A", "2"},
		/* 0x6x */ {"BIT", "4", "B", "2"}, {"BIT", "4", "C", "2"}, {"BIT", "4", "D", "2"}, {"BIT", "4", "E", "2"}, {"BIT", "4", "H", "2"}, {"BIT", "4", "L", "2"}, {"BIT", "4", "(HL)", "4"}, {"BIT", "4", "A", "2"}, {"BIT", "5", "B", "2"}, {"BIT", "5", "C", "2"}, {"BIT", "5", "D", "2"}, {"BIT", "5", "E", "2"}, {"BIT", "5", "H", "2"}, {"BIT", "5", "L", "2"}, {"BIT", "5", "(HL)", "4"}, {"BIT", "5", "A", "2"},
		/* 0x7x */ {"BIT", "6", "B", "2"}, {"BIT", "6", "C", "2"}, {"BIT", "6", "D", "2"}, {"BIT", "6", "E", "2"}, {"BIT", "6", "H", "2"}, {"BIT", "6", "L", "2"}, {"BIT", "6", "(HL)", "4"}, {"BIT", "6", "A", "2"}, {"BIT", "7", "B", "2"}, {"BIT", "7", "C", "2"}, {"BIT", "7", "D", "2"}, {"BIT", "7", "E", "2"}, {"BIT", "7", "H", "2"}, {"BIT", "7", "L", "2"}, {"BIT", "7", "(HL)", "4"}, {"BIT", "7", "A", "2"},

		/* 0x8x */ {"RES", "0", "B", "2"}, {"RES", "0", "C", "2"}, {"RES", "0", "D", "2"}, {"RES", "0", "E", "2"}, {"RES", "0", "H", "2"}, {"RES", "0", "L", "2"}, {"RES", "0", "(HL)", "4"}, {"RES", "0", "A", "2"}, {"RES", "1", "B", "2"}, {"RES", "1", "C", "2"}, {"RES", "1", "D", "2"}, {"RES", "1", "E", "2"}, {"RES", "1", "H", "2"}, {"RES", "1", "L", "2"}, {"RES", "1", "(HL)", "4"}, {"RES", "1", "A", "2"},
		/* 0x9x */ {"RES", "2", "B", "2"}, {"RES", "2", "C", "2"}, {"RES", "2", "D", "2"}, {"RES", "2", "E", "2"}, {"RES", "2", "H", "2"}, {"RES", "2", "L", "2"}, {"RES", "2", "(HL)", "4"}, {"RES", "2", "A", "2"}, {"RES", "3", "B", "2"}, {"RES", "3", "C", "2"}, {"RES", "3", "D", "2"}, {"RES", "3", "E", "2"}, {"RES", "3", "H", "2"}, {"RES", "3", "L", "2"}, {"RES", "3", "(HL)", "4"}, {"RES", "3", "A", "2"},
		/* 0xax */ {"RES", "4", "B", "2"}, {"RES", "4", "C", "2"}, {"RES", "4", "D", "2"}, {"RES", "4", "E", "2"}, {"RES", "4", "H", "2"}, {"RES", "4", "L", "2"}, {"RES", "4", "(HL)", "4"}, {"RES", "4", "A", "2"}, {"RES", "5", "B", "2"}, {"RES", "5", "C", "2"}, {"RES", "5", "D", "2"}, {"RES", "5", "E", "2"}, {"RES", "5", "H", "2"}, {"RES", "5", "L", "2"}, {"RES", "5", "(HL)", "4"}, {"RES", "5", "A", "2"},
		/* 0xbx */ {"RES", "6", "B", "2"}, {"RES", "6", "C", "2"}, {"RES", "6", "D", "2"}, {"RES", "6", "E", "2"}, {"RES", "6", "H", "2"}, {"RES", "6", "L", "2"}, {"RES", "6", "(HL)", "4"}, {"RES", "6", "A", "2"}, {"RES", "7", "B", "2"}, {"RES", "7", "C", "2"}, {"RES", "7", "D", "2"}, {"RES", "7", "E", "2"}, {"RES", "7", "H", "2"}, {"RES", "7", "L", "2"}, {"RES", "7", "(HL)", "4"}, {"RES", "7", "A", "2"},

		/* 0xcx */ {"SET", "0", "B", "2"}, {"SET", "0", "C", "2"}, {"SET", "0", "D", "2"}, {"SET", "0", "E", "2"}, {"SET", "0", "H", "2"}, {"SET", "0", "L", "2"}, {"SET", "0", "(HL)", "4"}, {"SET", "0", "A", "2"}, {"SET", "1", "B", "2"}, {"SET", "1", "C", "2"}, {"SET", "1", "D", "2"}, {"SET", "1", "E", "2"}, {"SET", "1", "H", "2"}, {"SET", "1", "L", "2"}, {"SET", "1", "(HL)", "4"}, {"SET", "1", "A", "2"},
		/* 0xdx */ {"SET", "2", "B", "2"}, {"SET", "2", "C", "2"}, {"SET", "2", "D", "2"}, {"SET", "2", "E", "2"}, {"SET", "2", "H", "2"}, {"SET", "2", "L", "2"}, {"SET", "2", "(HL)", "4"}, {"SET", "2", "A", "2"}, {"SET", "3", "B", "2"}, {"SET", "3", "C", "2"}, {"SET", "3", "D", "2"}, {"SET", "3", "E", "2"}, {"SET", "3", "H", "2"}, {"SET", "3", "L", "2"}, {"SET", "3", "(HL)", "4"}, {"SET", "3", "A", "2"},
		/* 0xex */ {"SET", "4", "B", "2"}, {"SET", "4", "C", "2"}, {"SET", "4", "D", "2"}, {"SET", "4", "E", "2"}, {"SET", "4", "H", "2"}, {"SET", "4", "L", "2"}, {"SET", "4", "(HL)", "4"}, {"SET", "4", "A", "2"}, {"SET", "5", "B", "2"}, {"SET", "5", "C", "2"}, {"SET", "5", "D", "2"}, {"SET", "5", "E", "2"}, {"SET", "5", "H", "2"}, {"SET", "5", "L", "2"}, {"SET", "5", "(HL)", "4"}, {"SET", "5", "A", "2"},
		/* 0xfx */ {"SET", "6", "B", "2"}, {"SET", "6", "C", "2"}, {"SET", "6", "D", "2"}, {"SET", "6", "E", "2"}, {"SET", "6", "H", "2"}, {"SET", "6", "L", "2"}, {"SET", "6", "(HL)", "4"}, {"SET", "6", "A", "2"}, {"SET", "7", "B", "2"}, {"SET", "7", "C", "2"}, {"SET", "7", "D", "2"}, {"SET", "7", "E", "2"}, {"SET", "7", "H", "2"}, {"SET", "7", "L", "2"}, {"SET", "7", "(HL)", "4"}, {"SET", "7", "A", "2"},
	}

	icon []byte = []byte{137, 80, 78, 71, 13, 10, 26, 10, 0, 0, 0, 13, 73, 72, 68, 82, 0, 0, 0, 32, 0, 0, 0, 32, 8, 6, 0, 0, 0, 115, 122, 122, 244, 0, 0, 0, 4, 115, 66, 73, 84, 8, 8, 8, 8, 124, 8, 100, 136, 0, 0, 0, 9, 112, 72, 89, 115, 0, 0, 14, 196, 0, 0, 14, 196, 1, 149, 43, 14, 27, 0, 0, 6, 9, 73, 68, 65, 84, 88, 133, 205,
		151, 89, 108, 19, 87, 20, 134, 255, 153, 241, 120, 137, 215, 196, 78, 216, 2, 33, 16, 32, 16, 168, 237, 76, 10, 73, 139, 128, 182, 168, 5, 26, 162, 96, 182, 22, 84, 250, 64, 27, 182, 7, 164, 46, 82, 43, 81, 241, 90, 30, 90, 218, 74, 21, 130, 7, 30, 104, 33, 15, 109, 165, 182, 44, 37, 108,
		21, 107, 33, 177, 147, 16, 2, 134, 176, 56, 118, 2, 222, 98, 59, 241, 62, 99, 103, 166, 15, 1, 7, 39, 142, 13, 18, 168, 61, 210, 72, 115, 239, 57, 62, 255, 119, 175, 206, 185, 115, 13, 252, 199, 70, 60, 239, 15, 74, 74, 74, 164, 186, 162, 137, 13, 154, 130, 252, 15, 102, 76, 159, 54, 143, 231, 57, 220, 127, 224, 184, 209, 31, 24, 248, 169, 207, 243, 232, 128, 221, 110, 143, 191, 12, 80, 0, 0, 195, 84, 87, 214, 214, 173, 234, 58, 241, 215, 113, 33, 145, 136, 10, 130, 192, 9, 44, 219, 43, 68, 34, 221, 194, 159, 71, 27, 133, 21, 117, 117, 93, 12, 83, 93, 249, 82, 196, 13,
		85, 213, 75, 183, 110, 223, 17, 238, 239, 239, 19, 4, 129, 75, 61, 44, 219, 155, 122, 60, 30, 171, 208, 176, 181, 33, 108, 168, 170, 94, 250, 66, 197, 25, 166, 186, 114, 203, 182, 29, 225, 120, 60, 148, 38, 62, 18, 128, 101, 123, 133, 96, 240, 129, 208, 176, 245, 227, 240, 11, 219, 9,
		189, 94, 175, 169, 55, 173, 181, 141, 92, 249, 88, 0, 44, 219, 43, 120, 189, 86, 161, 206, 100, 178, 233, 245, 122, 77, 174, 252, 100, 174, 0, 90, 166, 216, 187, 251, 171, 47, 166, 170, 213, 42, 132, 184, 48, 188, 177, 190, 156, 208, 42, 149, 18, 187, 190, 220, 62, 149, 150, 41, 246,
		230, 138, 165, 178, 57, 25, 102, 193, 66, 147, 169, 238, 251, 181, 107, 234, 9, 0, 144, 80, 98, 200, 233, 60, 132, 185, 48, 154, 236, 103, 112, 202, 113, 6, 23, 29, 221, 168, 158, 80, 6, 130, 24, 106, 168, 67, 183, 46, 98, 128, 141, 98, 193, 140, 217, 240, 249, 250, 245, 193, 129, 248, 89, 167, 243, 161, 99, 44, 141, 172, 59, 160, 208, 168, 191, 222, 182, 101, 115, 90, 171, 198, 146, 49, 236, 239, 60, 136, 107, 238, 22, 184, 163, 30, 152, 221, 54, 196, 7, 19, 41, 191, 35, 232, 131, 39, 26, 4, 0, 52, 124, 180, 158, 80, 168, 149, 123, 178, 105, 140, 9, 96, 168, 170, 94, 186, 110, 77, 253, 235, 98, 133, 24, 167, 28, 103, 145, 228, 147, 0, 128, 107, 174, 22, 4, 216, 64, 42, 142, 38, 41, 72, 41, 58, 53, 222, 85, 93, 143, 53, 51, 23, 0, 0, 52, 26, 21, 86, 155, 150, 189, 150, 173, 43, 198, 4, 200, 147, 201, 62, 89, 191, 206, 132, 88, 50, 6, 219, 64, 55, 18,
		252, 208, 42, 157, 17, 87, 122, 28, 45, 78, 109, 255, 211, 198, 11, 60, 66, 92, 12, 107, 87, 47, 67, 158, 76, 250, 233, 115, 1, 24, 141, 53, 37, 111, 44, 94, 248, 142, 182, 160, 0, 58, 153, 22, 91, 230, 109, 134, 76, 36, 3, 0, 168, 196, 170, 180, 88, 110, 48, 153, 49, 241, 31, 247, 44, 216, 249, 247, 33, 40, 213, 10, 44, 94, 52, 255, 109, 163, 177, 166, 228, 153, 1, 8, 74, 216, 184, 178, 118, 121, 70, 223, 252, 241, 12, 68, 132, 40, 53, 142, 37, 19, 96, 159, 170, 129, 39, 86, 51, 113, 38, 62, 172, 88, 4, 154, 18, 97, 197, 242, 37, 36, 65, 9, 27, 159, 25, 64, 167, 211, 153, 24, 198, 144, 54, 199, 14, 114, 0, 128, 66, 89, 33, 54, 205, 222, 0, 157, 84, 11, 0, 80, 137, 165, 160, 136, 209, 105, 198, 203, 213, 88, 92, 60, 27, 0, 80, 105, 172, 128, 78, 151, 111, 202, 164, 37, 26, 57, 97, 48, 24, 10, 25, 198, 96, 20, 137, 210, 93, 87, 93, 205, 88, 60, 105, 33, 0, 96, 154, 186, 20, 59, 13, 59, 16, 73, 70, 32, 36, 7, 32, 34, 135, 186, 249, 124, 175, 21, 22, 183, 13, 250, 194, 41, 120, 115, 114, 69, 170, 54, 68, 34, 10, 70, 99, 133, 209, 227, 114, 22, 182, 183, 183, 123, 179, 238, 0, 73, 75, 23, 50, 149, 250, 81, 243, 79, 196, 159, 24, 65, 16, 80, 208, 10, 72, 30, 119, 192, 169, 238, 14, 28, 232, 56, 7, 139, 219, 134, 131, 157, 231, 209, 120, 251, 74, 90, 188, 209, 80, 65, 146, 180, 52, 61, 73, 38, 0, 66, 128, 177, 188, 124, 214, 200, 233, 172, 230, 141, 6, 113, 100, 132, 224, 9, 219, 117, 120, 31, 159, 7, 0, 80, 62, 171, 20, 132, 0,
		99, 78, 0, 129, 64, 217, 148, 201, 197, 207, 5, 240, 219, 221, 102, 36, 248, 193, 244, 60, 16, 208, 209, 55, 124, 0, 22, 79, 158, 0, 129, 64, 89, 78, 0, 177, 84, 82, 164, 84, 42, 158, 89, 220, 27, 13, 226, 242, 163, 174, 140, 190, 64, 60, 146, 122, 87, 42, 228, 16, 75, 37, 69, 185, 1, 104, 154, 30, 57, 151, 205, 142, 219, 218, 193, 11, 66, 70, 159, 132, 74, 47, 100, 49, 37, 26, 149, 123, 20, 64, 52, 18, 11, 112, 28, 247, 76, 226, 209, 68, 20, 231, 123, 172, 99, 250, 139, 149, 218, 212, 59, 199, 37, 16, 137, 198, 251, 115, 2, 12, 242, 124, 87, 183, 61, 243, 199, 43, 196,
		133, 32, 60, 181, 218, 102, 183, 25, 28, 159, 249, 36, 148, 82, 52, 230, 20, 76, 74, 141, 237, 142, 135, 224, 5, 254, 78, 78, 0, 2, 184, 208, 98, 110, 27, 149, 208, 29, 245, 96, 143, 229, 91, 88, 3, 67, 57, 120, 129, 199, 53, 87, 75, 70, 113, 0, 88, 50, 121, 14, 36, 79, 237, 184, 165, 245, 38, 8, 224, 66, 78, 128, 96, 192, 123, 186, 169, 233, 140, 63, 204, 133, 17, 228, 134, 219, 72, 39, 213, 162, 182, 116, 57, 74, 85, 67, 71, 186, 61, 228, 64, 40, 17, 206, 40, 174, 145, 228, 97, 85, 89, 21, 0, 192, 31, 15, 99, 128, 141, 226, 244, 233, 75, 254, 96, 192, 123, 122, 100, 236, 168, 11, 137, 223, 239, 31, 20, 209, 82, 165, 87, 227, 93, 116, 147, 191, 5, 131, 238, 21, 208, 20, 13, 146, 32, 81, 172, 152, 4, 154, 28, 90, 149, 35, 212, 131, 91, 126, 43, 124, 145, 244, 2, 148, 82, 52, 62, 127, 181, 22, 227, 229, 26, 132, 185, 56, 118, 95, 249, 21, 231, 46, 53, 227, 122, 147, 229, 155, 155, 157, 29, 185, 1, 0, 64, 165, 156, 98, 238, 115, 186, 54, 21, 84, 106, 84, 110, 214, 131, 121, 218, 185, 32, 71, 156, 247, 106, 177, 10, 214, 192, 29, 244, 244, 15, 183, 90, 137, 74, 139, 207, 170, 222, 69, 169, 186, 8, 73, 126, 16, 223, 181, 158, 196, 253, 62, 23, 238, 54, 154, 31, 134, 220, 201, 13, 62, 95, 207, 168, 234, 206, 8, 224, 243, 245, 112, 90, 133, 174, 61, 234, 99, 55, 138, 103, 137, 72, 87, 204, 141, 217, 5, 229, 160, 136, 225, 112, 154, 164, 81, 85, 84, 137, 98, 121, 30, 202, 243, 39, 96, 229, 180, 74, 172, 159, 85, 3, 141, 84, 14, 110, 48, 137, 31, 218, 154, 208, 238, 182, 195, 123, 212, 154, 140, 247, 244, 175, 238, 236, 248, 103, 84, 1, 142, 9, 0, 0, 78, 231, 67, 155, 156, 200, 183, 199, 252, 241, 149, 68, 41, 200, 132, 192, 97, 102, 254, 140, 148, 223, 30, 116, 160, 47, 214, 7, 49, 65, 96, 130, 66, 131, 233, 154, 113, 169, 143, 207, 225, 219, 151, 113, 209, 110, 133, 247, 248, 157, 100, 168, 211, 189, 185, 205, 124, 245, 247, 177, 116, 178, 94, 74, 93, 206, 222, 235, 10, 34, 191, 45, 122, 63, 182, 172, 190, 166, 86, 54, 190, 104, 92, 202, 167, 145, 168, 161, 149, 105, 33, 2, 11, 133, 88, 10, 154, 28, 78, 197, 123, 98, 56, 185, 255, 184, 127, 160, 203, 243, 94, 155, 249, 234, 47, 217, 52, 178, 2, 0, 128, 243, 81, 111, 151, 74, 34, 255, 233, 216, 177, 179, 133, 247, 238, 63, 168, 144, 203, 229, 228, 184, 113, 133, 160, 40, 234, 113, 2, 22, 52, 73, 129, 227, 18, 104, 49, 223, 192, 190, 253, 71, 146, 251, 126, 60, 252, 115, 212, 231, 55, 181, 154, 155, 91, 115, 229, 127, 174, 63, 167, 12, 195, 76, 225, 73, 241, 251, 178, 60, 233, 91, 211, 167, 78, 157, 171, 82, 169, 10, 121, 129, 67, 40, 24, 246, 62, 176, 245, 118, 198, 226, 241, 179, 36, 207,
		53, 90, 44, 150, 49, 175, 225, 255, 59, 251, 23, 242, 156, 177, 101, 178, 79, 133, 255, 0, 0, 0, 0, 73, 69, 78, 68, 174, 66, 96, 130}
)
